
/*
 * Licensed Materials - Property of IBM
 *
 * trousers - An open source TCG Software Stack
 *
 * (C) Copyright International Business Machines Corp. 2004-2006
 *
 */

#ifndef _TSS_H_
#define _TSS_H_

#include <stdlib.h>

#include <tss/platform.h>
#include <tss/tss_error_basics.h>
#include <tss/tddl_error.h>
#include <tss/tcs_typedef.h>
#include <tss/tss_typedef.h>
#include <tss/tss_structs.h>
#include <tss/tcs_structs.h>
#include <tss/tcs_error.h>
#include <tss/tcs.h>
#include <tss/tss_error.h>

#include <tss/tddlapi_error.h>
#include <tss/tcs_defines.h>
#include <tss/tss_defines.h>
#include <tss/tddli.h>
#include <tss/tspi.h>

#endif
